package com.uas.plugins.maven.ops.diff.plugin;

import com.uas.ops.diff.domain.Diff;
import com.uas.ops.diff.domain.Manifest;
import com.uas.plugins.maven.ops.diff.provider.ManifestService;
import com.uas.plugins.maven.ops.diff.util.ObjectUtils;
import com.uas.plugins.maven.ops.diff.util.ZipUtils;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.apache.maven.plugins.annotations.LifecyclePhase;
import org.apache.maven.plugins.annotations.Mojo;
import org.codehaus.plexus.util.FileUtils;

import java.io.File;
import java.io.IOException;

@Mojo(name = "diff", defaultPhase = LifecyclePhase.INSTALL, threadSafe = true)
public class DiffMojo extends MyAbstractMojo {

	private ManifestService manifestService;

	@Override
	public void execute() throws MojoExecutionException, MojoFailureException {
		if (this.skip) {
			getLog().info("Skipping create diff");
		} else {
            File newManifestFile = getManifestFile();
            if (null != newManifestFile && newManifestFile.exists()) {
                manifestService = new ManifestService(this.repository, artifact.getGroupId(), artifact.getArtifactId());
                try {
                    String lastVersion = manifestService.getLastVersion();
                    Manifest newManifest = ObjectUtils.read(newManifestFile);
                    if (!newManifest.getVersion().equals(lastVersion)) {
                        if (null != lastVersion && !lastVersion.isEmpty()) {
                            File lastManifestFile = getLastManifestFile();
                            manifestService.downloadManifest(lastVersion, lastManifestFile);
                            Manifest lastManifest = ObjectUtils.read(lastManifestFile);
                            Diff diff = new Diff(lastManifest, newManifest);
                            createDiffFile(diff);
                        } else {
                            ZipUtils.zip(new File(targetdir, buildName), getDiffFile());
                        }
                        getLog().info("Diff [" + getDiffFile().getAbsolutePath() + "] created");
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
		}
	}

	private void createDiffFile(Diff diff) throws IOException{
        if (!diffTarget.exists()) {
            diffTarget.mkdirs();
        }
        File sourceDir = new File(targetdir, buildName);
        for (String path:diff.getFileList()) {
            getLog().info("DIFF FILE: " + path);
            File file = new File(sourceDir, path);
            if (file.exists()) {
                File targetFile = new File(diffTarget, path);
                if (file.isDirectory()) {
                    targetFile.mkdirs();
                    FileUtils.copyDirectory(file, targetFile.getParentFile());
                } else {
                    FileUtils.copyFile(file, targetFile);
                }
            }
        }
        ZipUtils.zip(diffTarget, getDiffFile());
    }
}
