package com.uas.plugins.maven.ops.diff.plugin;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import com.uas.ops.diff.domain.Manifest;
import com.uas.plugins.maven.ops.diff.util.ObjectUtils;
import com.uas.plugins.maven.ops.diff.util.SvnUtils;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.apache.maven.plugins.annotations.LifecyclePhase;
import org.apache.maven.plugins.annotations.Mojo;
import org.apache.maven.plugins.annotations.Parameter;
import org.codehaus.plexus.util.MatchPatterns;
import org.tmatesoft.svn.core.wc2.SvnStatus;

/**
 * 创建版本信息清单
 * 
 * @author yingp
 *
 */
@Mojo(name = "manifest", defaultPhase = LifecyclePhase.INSTALL, threadSafe = true)
public class ManifestMojo extends MyAbstractMojo {

	@Parameter(property = "packagingIncludes", alias = "includes")
	private ArrayList<String> packagingIncludes;

	@Parameter(property = "packagingExcludes", alias = "excludes")
	private ArrayList<String> packagingExcludes;

    @Parameter(property = "sourceDir", defaultValue = "src/main/java")
    private String sourceDir;

    @Parameter(property = "resourceDir", defaultValue = "src/main/resources")
    private String resourceDir;

    @Parameter(property = "webappSourceDir", defaultValue = "src/main/websrc")
    private String webappSourceDir;

	private MatchPatterns includesPatterns;

	private MatchPatterns excludesPatterns;

	@Override
	public void execute() throws MojoExecutionException, MojoFailureException {
		if (this.skip) {
			getLog().info("Skipping create manifest");
		} else {
			if (null == packagingIncludes || packagingIncludes.isEmpty()) {
				packagingIncludes = new ArrayList<String>(Collections.singletonList("**"));
			}
			if (null == packagingExcludes) {
				packagingExcludes = new ArrayList<String>();
			}
			this.includesPatterns = MatchPatterns.from(packagingIncludes);
			this.excludesPatterns = MatchPatterns.from(packagingExcludes);
			try {
				File manifestFile = getManifestFile();
				if (isSubversion()) {
					Manifest manifest = getManifest(basedir, SvnUtils.getTargetStatus(basedir));
					ObjectUtils.write(manifest, manifestFile);
					getLog().info("Manifest [" + manifestFile.getAbsolutePath() + "] created");
				} else if(isGit()) {

                } else {
					getLog().warn("UnSupported version control type, Skipping create manifest");
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	private Manifest getManifest(File file, Map<File, SvnStatus> statusMap) throws IOException {
		SvnStatus status = statusMap.get(file);
		Manifest manifest = null;
		if (null != status && includesPatterns.matches(status.getRepositoryRelativePath(), true)
				&& !excludesPatterns.matches(status.getRepositoryRelativePath(), true)) {
			manifest = new Manifest(parseRepositoryPath(status.getRepositoryRelativePath()), String.valueOf(status.getChangedRevision()), status.getChangedDate().getTime());
			if (file.isDirectory()) {
				List<Manifest> children = new ArrayList<Manifest>();
				for (File f : file.listFiles()) {
					Manifest child = getManifest(f, statusMap);
					if (null != child)
						children.add(child);
				}
				manifest.setChildren(children);
			}
		}
		return manifest;
	}

    private String parseRepositoryPath(String path) {
        if (path.contains("/")) {
            // remove repositoryBasePath
            String subpath = path.substring(path.indexOf("/") + 1);
            // replace {sourceDir} into {targetDir}/{buildName}/WEB-INF/classes
            if (subpath.equals(sourceDir) || subpath.matches(sourceDir + "/.*")) {
                subpath = subpath.replaceFirst(sourceDir, "WEB-INF/classes");
            }
            // replace {resourceDir} into {targetDir}/{buildName}/WEB-INF
            if (subpath.equals(resourceDir) || subpath.matches(resourceDir + "/.*")) {
                subpath = subpath.replaceFirst(resourceDir, "WEB-INF/classes");
            }
            // replace {webappSourceDir} into {targetDir}/{buildName}
            if (subpath.equals(webappSourceDir)) {
                subpath = subpath.replaceFirst(webappSourceDir, "");
            } else if (subpath.matches(webappSourceDir + "/.*")) {
                subpath = subpath.replaceFirst(webappSourceDir + "/", "");
            }
            // .java build to .class
            if (subpath.endsWith(".java")) {
                subpath = subpath.substring(0, subpath.lastIndexOf(".java")) + ".class";
            }
            path = "/" + subpath;
        } else {
            path = "/";
        }
        return path;
    }

}
