package com.uas.plugins.maven.ops.diff.provider;

import com.uas.plugins.maven.ops.diff.util.HttpUtils;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by Pro1 on 2017/6/15.
 */
public class ManifestService {

    private final String versionApi;
    private final String manifestApi;
    private final String groupId;
    private final String artifactId;

    public ManifestService(String repositoryUrl, String groupId, String artifactId) {
        this.versionApi = repositoryUrl + "/version";
        this.manifestApi = repositoryUrl + "/manifest";
        this.groupId = groupId;
        this.artifactId = artifactId;
    }

    public String getLastVersion() throws Exception {
        Map<String, Object> data = new HashMap<String, Object>();
        data.put("groupId", this.groupId);
        data.put("artifactId", this.artifactId);
        HttpUtils.ResponseWrap resp = HttpUtils.get(this.versionApi, data);
        if (resp.isSuccess()) {
            return resp.getContent();
        }
        throw new Exception(resp.getContent());
    }

    public void downloadManifest(String version, File targetFile) throws Exception {
        Map<String, Object> data = new HashMap<String, Object>();
        data.put("groupId", this.groupId);
        data.put("artifactId", this.artifactId);
        data.put("version", version);
        HttpUtils.ResponseWrap resp = HttpUtils.download(this.manifestApi, data, targetFile);
        if (!resp.isSuccess()) {
            throw new Exception(resp.getContent());
        }
    }

    public void uploadManifest(File sourceFile) throws Exception {
        Map<String, Object> data = new HashMap<String, Object>();
        data.put("groupId", this.groupId);
        data.put("artifactId", this.artifactId);
        HttpUtils.ResponseWrap resp = HttpUtils.upload(this.manifestApi, data, sourceFile);
        if (!resp.isSuccess()) {
            throw new Exception(resp.getContent());
        }
    }

}
