package com.uas.plugins.test;

import com.uas.ops.diff.domain.Manifest;
import com.uas.plugins.maven.ops.diff.util.ObjectUtils;
import com.uas.plugins.maven.ops.diff.util.SvnUtils;
import org.codehaus.plexus.util.MatchPatterns;
import org.tmatesoft.svn.core.wc2.SvnStatus;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * Created by Pro1 on 2017/6/15.
 */
public class ManifestTest {

    static final File basedir = new File("C:\\Source\\1.0.0");

    static MatchPatterns includesPatterns = MatchPatterns.from(new String[]{ "**" });

    static MatchPatterns excludesPatterns = MatchPatterns.from(new String[]{"**/.classpath", "**/.project", "**/.settings", "**/build", "**/pom.xml"});

    static String sourceDir = "src";

    static String resourceDir = "resources";

    static String webappSourceDir = "WebContent";

    public static void main(String[] args) throws Exception{
        Manifest manifest = getManifest(basedir, SvnUtils.getTargetStatus(basedir));
        File manifestFile = new File(basedir, "target\\.manifest");
        ObjectUtils.write(manifest, manifestFile);
        manifest.println(new PrintStream(new FileOutputStream(new File(manifestFile.getAbsolutePath() + ".txt"))));
    }

    private static Manifest getManifest(File file, Map<File, SvnStatus> statusMap) throws IOException {
        SvnStatus status = statusMap.get(file);
        Manifest manifest = null;
        if (null != status && includesPatterns.matches(status.getRepositoryRelativePath(), true)
                && !excludesPatterns.matches(status.getRepositoryRelativePath(), true)) {
            manifest = new Manifest(parseRepositoryPath(status.getRepositoryRelativePath()), String.valueOf(status.getChangedRevision()), status.getChangedDate().getTime());
            if (file.isDirectory()) {
                List<Manifest> children = new ArrayList<Manifest>();
                for (File f : file.listFiles()) {
                    Manifest child = getManifest(f, statusMap);
                    if (null != child)
                        children.add(child);
                }
                manifest.setChildren(children);
            }
        }
        return manifest;
    }

    private static String parseRepositoryPath(String path) {
        if (path.contains("/")) {
            // remove repositoryBasePath
            String subpath = path.substring(path.indexOf("/") + 1);
            // replace {sourceDir} into {targetDir}/{buildName}/WEB-INF/classes
            if (subpath.equals(sourceDir) || subpath.matches(sourceDir + "/.*")) {
                subpath = subpath.replaceFirst(sourceDir, "WEB-INF/classes");
            }
            // replace {resourceDir} into {targetDir}/{buildName}/WEB-INF
            if (subpath.equals(resourceDir) || subpath.matches(resourceDir + "/.*")) {
                subpath = subpath.replaceFirst(resourceDir, "WEB-INF/classes");
            }
            // replace {webappSourceDir} into {targetDir}/{buildName}
            if (subpath.equals(webappSourceDir)) {
                subpath = subpath.replaceFirst(webappSourceDir, "");
            } else if (subpath.matches(webappSourceDir + "/.*")) {
                subpath = subpath.replaceFirst(webappSourceDir + "/", "");
            }
            // .java build to .class
            if (subpath.endsWith(".java")) {
                subpath = subpath.substring(0, subpath.lastIndexOf(".java")) + ".class";
            }
            path = "/" + subpath;
        } else {
            path = "/";
        }
        return path;
    }

}
